//
//  WDataPicker.m
//  Dealloc
//
//  Created by frank on 11-7-13.
//  Copyright wongf70@gmail.com All rights reserved.
//

#import "WDataPicker.h"
#import <UIKit/UIKit.h>
#import <QuartzCore/QuartzCore.h>

//interface----------------------------------------------------------------
@interface WDragLabel : UIView {
	CGPoint startPoint; 
	CGPoint currentPoint;
	CGPoint lastPoint;
	CGPoint endPoint;
	CGFloat velocity;
	NSTimer*timer;
	int length;
	float width;
	float height;
	float sectionWidth;
}
- (id)initWithFrame:(CGRect)frame array:(NSArray*)list row:(int)aRow;
-(int)index;
@end

//implemetation ----------------------------------------------------------------
@implementation WDragLabel

- (id)initWithFrame:(CGRect)frame array:(NSArray*)list row:(int)aRow{
    
    if (list&&(aRow==1||aRow==3||aRow==5||aRow==7))
	{
		height=frame.size.height;
		width=frame.size.width;
		sectionWidth=frame.size.width/aRow;
		length=[list count];
		self = [super initWithFrame:CGRectMake(0, 0, sectionWidth*length, height)];
		if (!((int)length%2))
		{
			self.center=CGPointMake((width-sectionWidth)/2, self.center.y);
		}else 
		{
			self.center=CGPointMake(width/2, self.center.y);
		}
		
		for (int i=0; i<[list count]; i++)
		{
			NSObject*obj=[list objectAtIndex:i];
			if ([obj isKindOfClass:[NSString class]]) 
			{
				UILabel*label=[[UILabel alloc]initWithFrame:CGRectMake(sectionWidth*i, 0, sectionWidth, height)];
				label.font=[UIFont systemFontOfSize:height/2];
				label.textAlignment=UITextAlignmentCenter;
				label.text=[list objectAtIndex:i];
				[self addSubview:label];
				label.backgroundColor=[UIColor clearColor];
				label.alpha=0.5;
			}else if ([obj isKindOfClass:[UIImage class]])
			{
				UIImageView*imageView=[[UIImageView alloc]initWithImage:(UIImage*)obj];
				imageView.frame=CGRectMake(sectionWidth*i, 0, sectionWidth, height);
				[self addSubview:imageView];
			}
		}
	}
    return self;
}
-(void)animationAfterDeceleration
{
	//[UIView animateWithDuration:0.3 animations:^(void) {self.center=CGPointMake(280-sectionWidth*[self index],self.center.y);}];
	[UIView animateWithDuration:0.3 animations:^(void) {self.center=CGPointMake(sectionWidth*(length/2-[self index]-(length%2?0:0.5))+width/2,self.center.y);}];
	
}
-(void)moveLimiting
{
	if (self.center.x<1-length*sectionWidth/2+width/2)
	{
		self.center=CGPointMake(1-length*sectionWidth/2+width/2, self.center.y);
		velocity*=-1;
	}else if (self.center.x>length*sectionWidth/2+width/2-1) 
	{
		self.center=CGPointMake(length*sectionWidth/2+width/2-1, self.center.y);
		velocity*=-1;
	}
}
-(void) decelerate
{
	float maxSpeed=28.0;
	velocity=velocity<-maxSpeed?-maxSpeed:velocity;
	velocity=velocity>maxSpeed?maxSpeed:velocity;
	if (fabs(velocity*=0.9)<0.06&&[timer isValid])
	{
		[timer invalidate];
		timer=nil;
		[self animationAfterDeceleration];
		return;
	}
	self.center=CGPointMake(self.center.x-velocity,self.center.y);
	[self moveLimiting];
}
- (void) touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event
{	
	if ([timer isValid]) 
	{
		[timer invalidate];
		timer=nil;
	}
	[self.layer removeAllAnimations];
	startPoint = [[touches anyObject] locationInView:self];
	lastPoint=startPoint;
}
- (void) touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event
{
	currentPoint = [[touches anyObject] locationInView:self];
	lastPoint=currentPoint;
	self.center=CGPointMake(self.center.x+currentPoint.x-startPoint.x,self.center.y);
	[self moveLimiting];
}
-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	UITouch*endTouch=[touches anyObject];
	endPoint=[endTouch locationInView:self];
	velocity=endPoint.x-lastPoint.x;
	if (fabs(velocity)<0.06||[event timestamp]-[endTouch timestamp]>0.3) 
	{
		[self animationAfterDeceleration];
		return;
	}
	timer=[NSTimer scheduledTimerWithTimeInterval:0.01 target:self selector:@selector(decelerate) userInfo:nil repeats:YES];
}
-(int)index
{
	return (width/2-self.center.x)/sectionWidth+length/2+(length%2?0.5:0);
}
- (void)dealloc 
{
	if ([timer isValid]) 
	{
		[timer invalidate];
		timer=nil;
	}
}
@end


//implementation ---------------------------------------------------------------- 
@implementation WDataPicker

- (id)initWithFrame:(CGRect)frame array:(NSArray*)list row:(int)aRow{
    
	self = [super initWithFrame:frame];
    if (self) 
	{
		self.layer.masksToBounds=YES;
		UIImageView*imageView=[[UIImageView alloc]initWithImage:[UIImage imageNamed:@"bg.png"]];
		imageView.frame=CGRectMake(0, 0, frame.size.width, frame.size.height);
		[self addSubview:imageView];
		[self sendSubviewToBack:imageView];
	
		scrollBar=[[WDragLabel alloc]initWithFrame:frame array:list row:aRow];
		[self addSubview:scrollBar];
    }
    return self;
}
-(int)index
{
	return [scrollBar index];
}


@end
